// VString.cpp: implementation of the VString class.
//
//////////////////////////////////////////////////////////////////////

#include "globalf.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "vstring.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

VString::VString()
{
	data = new char[1025];
	if (data == NULL) { MemError("constructor"); }
	arraylength = 1024;
	ourlength = 0;
	strcpy(data, "");
}

VString::~VString()
{
		delete [] data;
}

void VString::Set(char *input)
{
	// int length;

	delete [] data; // Delete the previous string first!

	ourlength = strlen(input); // Get the string length
	arraylength = 1024 * (1 + ourlength / 1024);
	data = new char[arraylength+1]; // Make a new string array
	if (data == NULL) { MemError("set"); }
	strcpy(data, input); // Copy the input string into the new string
}

char *VString::Get()
{
	return data;
}


int VString::Instr(char *search, int start)
{
	// Look for the VB descriptions of this function for workings!
	// Returns -1 on error (not found)
	// Start is 0, not 1!

	//int stringlen = strlen(data);
	int searchlen = strlen(search);

	int i, j;
	int pos;
	bool found = false;

	if (searchlen > ourlength || start >= ourlength || start < 0) { return -1; }

	//cout << "TextLen: " << textlen << "\n";
	//cout << "SeekLen: " << seeklen << "\n";
	//cout.flush();

	pos = start;

	while (!found)
	{
		for (i = pos; i < ourlength; i++)
		{
			//cout << "Pos: " << i << "\n";
			//cout.flush();
			if (data[i] == search[0])
			{
				break;
			}
		}
		if (i >= ourlength) { return -1; }
		
		pos = i;
		j = 0;

		//cout << "Found?\n";
		//cout.flush();

		for (i = pos; i < searchlen + pos; i++)
		{
			//cout << "Pos: " << i << "\n";
			//cout.flush();
			if (i >= ourlength)
			{
				return -1;
			}

			if (data[i] == search[j])
			{
				found = true;
				j++;
			}
			else
			{
				found = false;
				pos++;
				break;
			}
		}

	}

	return pos;
}

int VString::InstrRev(char *search, int start)
{
	// This fucntion does a reverse Instr. That means it starts searching at the END of the string!
	// Look for the VB descriptions of this function for workings!
	// Returns -1 on error (not found)
	// Start is 0, not 1!

	int i, j;
	int iTemp;

	i = ourlength - 1;
	iTemp = strlen(search) - 1;

	if (start > i) { return -1; }

	for (i = start - iTemp; i >= 0; i--)
	{
		for (j = iTemp; j >= 0 && search[j] == data[i + j]; j--)
		{
			// do nothing
		}
		if (search[j] == 0)
		{
			// Entire search string walked: FOUND STRING!
			return i;
		}
		if (data[i + j] == 0)
		{
			// Reached end of string
			return -1;
		}
	}

	return -1;
}

void VString::Mid(VString *output, int start, int length)
{
	//int stringlen = strlen(data);
	int i, j;

	char *temp;
	// cout << length;
	// cout.flush();

	if (start > ourlength)
	{
		output->Set("");
		return;
	}
	if (length < 0)
	{
		output->Set("");
		return;
	}
	if ((start + length) > ourlength) { length = ourlength - start; }

	output->arraylength = 1024 * (1 + length / 1024);

	temp = new char[output->arraylength+1];
	if (temp == NULL) { MemError("mid", length); }
	
	j = 0;
	for (i = start; i < start + length; i++)
	{
		// cout << text[i];
		// cout.flush();

		temp[j] = data[i];
		j++;
	}

	temp[j] = 0;
	output->Give(temp, length);

	return;
}

void VString::Left(VString *output, int length)
{
	//int stringlen = strlen(data);
	int i, j;

	char *temp;
	// cout << length;
	// cout.flush();

	if (length < 0)
	{
		output->Set("");
		return;
	}
	if (length > ourlength) { length = ourlength; }

	output->arraylength = 1024 * (1 + length / 1024);

	temp = new char[output->arraylength+1];
	if (temp == NULL) { MemError("left"); }
	
	j = 0;
	for (i = 0; i < length; i++)
	{
		// cout << text[i];
		// cout.flush();

		temp[j] = data[i];
		j++;
	}

	temp[j] = 0;
	output->Give(temp, length);

	return;
}

void VString::Right(VString *output, int length)
{
	//int stringlen = strlen(data);
	int i, j;
	int start;

	char *temp;
	// cout << length;
	// cout.flush();
	if (length < 0)
	{
		output->Set("");
		return;
	}
	if (length > ourlength) { length = ourlength; }

	output->arraylength = 1024 * (1 + length / 1024);

	temp = new char[output->arraylength+1];
	if (temp == NULL) { MemError("right"); }

	start = (ourlength - length);
	
	j = 0;
	for (i = start; i < ourlength; i++)
	{
		// cout << text[i];
		// cout.flush();

		temp[j] = data[i];
		j++;

	}

	temp[j] = 0;

	output->Give(temp, length);

	return;
}

int VString::GetLength()
{
	return ourlength;
}

void VString::Cat(char *addition)
{
	// int ourlength;
	char *temp;

	ourlength += strlen(addition);

	arraylength = 1024 * (1 + ourlength / 1024);

	temp = new char[arraylength+1];
	if (temp == NULL) { MemError("cat"); }

	strcpy(temp, data);
	strcat(temp, addition);

	delete [] data;

	data = temp;
}

void VString::Give(char *gift, int giftsize)
{
	delete [] data;

	data = gift;
	ourlength = giftsize;
}

void VString::MemError(char *error, int errorno)
{
	printf("\n\nMemory allocation error (VString - %s - %i).\n", error, errorno);
	printf("\nTry to free more memory on your computer\n\n");
	printf("If this fails contact resgen@hltools.com\n");
	exit(2);
}

void VString::LCase()
{
	// Converts the data to lowercase
	data = strlwr(data);
}

void VString::Trim(char trimchar)
{
	// trims any leading spaces and ending spaces
	LTrim(trimchar);
	RTrim(trimchar);
}

void VString::LTrim(char trimchar)
{
	// trims the leading spaces
	//int datalen = strlen(data);

	int pos = 0;
	while (data[pos] == trimchar && pos < ourlength)
	{
		pos++; // Increase to check next char
	}
	if (pos > 0) { Right(this, ourlength - pos); }
}

void VString::RTrim(char trimchar)
{
	// trims the trailing spaces
	int pos = ourlength - 1;

	while (data[pos] == trimchar && pos >= 0)
	{
		pos--; // Decrease to check previous char.
	}

	if (pos < ourlength - 1) { Left(this, pos + 1); }
}